<?php
/**
 * The settings.
 *
 * @package   BlockEnhancements
 * @author    Phi Phan <mrphipv@gmail.com>
 * @copyright Copyright (c) 2022, Phi Phan
 */

namespace BlockEnhancements;

// Exit if accessed directly.
defined( 'ABSPATH' ) || exit;

if ( ! class_exists( Settings::class ) ) :
	/**
	 * The controller class for the settings.
	 */
	class Settings extends CoreComponent {
		/**
		 * Setting page's hook suffix.
		 *
		 * @var string
		 */
		protected $hook_suffix;

		/**
		 * Run main hooks
		 *
		 * @return void
		 */
		public function run() {
			// Create the settings page.
			add_action( 'admin_menu', [ $this, 'add_admin_page' ] );

			// Enqueue settings script.
			add_action( 'admin_enqueue_scripts', [ $this, 'enqueue_settings_scripts' ] );

			// Register settings.
			add_action( 'init', [ $this, 'register_settings' ] );

			// Add the settings page link to plugin list screen.
			add_action( 'plugin_action_links_' . plugin_basename( BLOCK_ENHANCEMANCES_ROOT_FILE ), [ $this, 'plugin_settings_links' ] );

			// Change the footer text for the settings pages.
			add_action( 'admin_footer_text', [ $this, 'admin_footer_text' ] );
		}

		/**
		 * Create the admin page
		 *
		 * @return void
		 */
		public function add_admin_page() {
			$this->hook_suffix = add_options_page(
				__( 'Block Enhancements' ),
				__( 'Block Enhancements' ),
				'manage_options',
				'be-settings',
				function () {
					?>
					<div class="wrap be-settings js-be-settings-root">
						<div class="be-settings__header">
							<h1><?php printf( __( '<strong>Block Enhancements</strong>' ) ); ?> <code><?php echo esc_html( $this->the_plugin_instance->get_plugin_version() ); ?></code></h1>
						</div>
						<div class="be-settings__body"></div>
						<div class="be-settings__footer"></div>
					</div>
					<?php
				}
			);
		}

		/**
		 * Enqueue scripts for the settings page.
		 *
		 * @param string $hook_suffix
		 * @return void
		 */
		public function enqueue_settings_scripts( $hook_suffix ) {
			// Only load scripts for the settings page.
			if ( 'settings_page_be-settings' === $hook_suffix ) {
				// Load all registered blocks and categories.
				$block_categories = array();
				if ( function_exists( 'gutenberg_get_block_categories' ) ) {
					$block_categories = gutenberg_get_block_categories( get_post() );
				} elseif ( function_exists( 'get_block_categories' ) ) {
					$block_categories = get_block_categories( get_post() );
				}

				wp_add_inline_script(
					'wp-blocks',
					sprintf( 'wp.blocks.setCategories( %s );', wp_json_encode( $block_categories ) ),
					'after'
				);

				$block_registry = \WP_Block_Type_Registry::get_instance();
				foreach ( $block_registry->get_all_registered() as $block_name => $block_type ) {
					// Front-end script.
					if ( ! empty( $block_type->editor_script ) ) {
						wp_enqueue_script( $block_type->editor_script );
					}
				}

				// Settings asset file.
				$settings_asset = $this->the_plugin_instance->include_file( 'build/settings.asset.php' );

				// Enqueue scripts.
				wp_enqueue_script(
					'be-settings',
					$this->the_plugin_instance->get_file_uri( 'build/settings.js' ),
					$settings_asset['dependencies'] ?? [],
					$this->the_plugin_instance->get_script_version( $settings_asset ),
					true
				);

				wp_set_script_translations( 'be-settings', 'block-enhancements' );

				// Block features.
				wp_localize_script(
					'be-settings',
					'BlockEnhancementsFeatures',
					$this->the_plugin_instance->get_features()
				);

				// Load components style.
				wp_enqueue_style( 'wp-components' );
			}
		}

		/**
		 * Register settings.
		 */
		public function register_settings() {
			// Setting fields.
			register_setting(
				'be',
				'be_allowed_blocks',
				[
					'type'         => 'array',
					'show_in_rest' => array(
						'schema' => array(
							'items' => array(
								'type'       => 'object',
								'properties' => array(
									'featureName'   => array(
										'type' => 'string',
									),
									'allowedBlocks' => array(
										'type'  => 'array',
										'items' => array(
											'type'       => 'object',
											'properties' => array(
												'name' => array(
													'type' => 'string',
												),
											),
										),
									),
								),
							),
						),
					),
					'default'      => [
						[
							'featureName'   => 'withIcon',
							'allowedBlocks' => [
								[ 'name' => 'core/button' ],
								[ 'name' => 'core/read-more' ],
								[ 'name' => 'core/list' ],
								[ 'name' => 'core/heading' ],
							],
						],
						[
							'featureName'   => 'withColor',
							'allowedBlocks' => [
								[ 'name' => 'core/button' ],
								[ 'name' => 'core/read-more' ],
								[ 'name' => 'boldblocks/svg-block' ],
							],
						],
						[
							'featureName'   => 'withShadow',
							'allowedBlocks' => [
								[ 'name' => 'core/button' ],
								[ 'name' => 'core/read-more' ],
								[ 'name' => 'core/image' ],
								[ 'name' => 'core/group' ],
								[ 'name' => 'core/columns' ],
								[ 'name' => 'core/column' ],
								[ 'name' => 'core/media-text' ],
								[ 'name' => 'core/cover' ],
								[ 'name' => 'boldblocks/svg-block' ],
							],
						],
						[
							'featureName'   => 'withTextShadow',
							'allowedBlocks' => [
								[ 'name' => 'core/heading' ],
								[ 'name' => 'core/paragraph' ],
								[ 'name' => 'core/group' ],
								[ 'name' => 'core/columns' ],
								[ 'name' => 'core/column' ],
								[ 'name' => 'core/media-text' ],
								[ 'name' => 'core/cover' ],
							],
						],
						[
							'featureName'   => 'withTransform',
							'allowedBlocks' => [
								[ 'name' => 'core/group' ],
								[ 'name' => 'core/columns' ],
								[ 'name' => 'core/column' ],
								[ 'name' => 'core/media-text' ],
								[ 'name' => 'core/cover' ],
								[ 'name' => 'core/button' ],
								[ 'name' => 'core/read-more' ],
								[ 'name' => 'core/image' ],
								[ 'name' => 'core/heading' ],
								[ 'name' => 'core/paragraph' ],
								[ 'name' => 'boldblocks/svg-block' ],
							],
						],
						[
							'featureName'   => 'withTransition',
							'allowedBlocks' => [
								[ 'name' => 'core/group' ],
								[ 'name' => 'core/columns' ],
								[ 'name' => 'core/column' ],
								[ 'name' => 'core/media-text' ],
								[ 'name' => 'core/cover' ],
								[ 'name' => 'core/button' ],
								[ 'name' => 'core/read-more' ],
								[ 'name' => 'core/image' ],
								[ 'name' => 'core/heading' ],
								[ 'name' => 'core/paragraph' ],
								[ 'name' => 'boldblocks/svg-block' ],
							],
						],
					],
				]
			);

			// Responsive settings.
			register_setting(
				'be',
				'be_breakpoints',
				[
					'type'         => 'object',
					'show_in_rest' => array(
						'schema' => array(
							'properties' => array(
								'sm' => array(
									'type'       => 'object',
									'properties' => array(
										'breakpoint' => array(
											'type' => 'string',
										),
										'mediaQuery' => array(
											'type' => 'string',
										),
									),
								),
								'md' => array(
									'type'       => 'object',
									'properties' => array(
										'breakpoint' => array(
											'type' => 'string',
										),
										'mediaQuery' => array(
											'type' => 'string',
										),
									),
								),
								'lg' => array(
									'type'       => 'object',
									'properties' => array(
										'breakpoint' => array(
											'type' => 'string',
										),
										'mediaQuery' => array(
											'type' => 'string',
										),
									),
								),
							),
						),
					),
					'default'      => [
						'sm' => [
							'breakpoint' => '576px',
							'mediaQuery' => '',
						],
						'md' => [
							'breakpoint' => '768px',
							'mediaQuery' => '@media (min-width: 768px){##CONTENT##}',
						],
						'lg' => [
							'breakpoint' => '1024px',
							'mediaQuery' => '@media (min-width: 1024px){##CONTENT##}',
						],
					],
				]
			);
		}

		/**
		 * Add the settings page link to the plugin admin screen.
		 *
		 * @param array $links
		 * @return array
		 */
		public function plugin_settings_links( $links ) : array {
			$label = esc_html__( 'Settings', 'block-enhancements' );
			$slug  = 'be-settings';

			array_unshift( $links, "<a href='options-general.php?page=$slug'>$label</a>" );
			return $links;
		}

		/**
		 * Change the footer text for the settings pages
		 *
		 * @param string $footer_text
		 * @return string
		 */
		public function admin_footer_text( $footer_text ) {
			// Get current screen.
			$current_screen = get_current_screen();
			if ( 'settings_page_be-settings' === $current_screen->id ) {
				$footer_text = '<i><strong>Block Enhancements</strong> <code>' . esc_html__( $this->the_plugin_instance->get_plugin_version() ) . '</code>. Please <a target="_blank" href="https://wordpress.org/support/plugin/block-enhancements/reviews/#new-post" title="Rate the plugin" style="text-decoration:none">rate the plugin <span style="color:#ffb900">★★★★★</span></a> to help us spread the word. Thank you from the <a href="https://boldblocks.net/?utm_source=Block+Enhancements&utm_campaign=Block+Enhancements+visit+site&utm_medium=link&utm_content=footer-text" target="_blank" title="' . __( 'Visit the Plugin website', 'block-enhancements' ) . '" style="text-decoration:none"><strong>BoldBlocks</strong></a> team!</i>';
			}

			return $footer_text;
		}
	}
endif;
